#include <avr/io.h>
#include <util/delay.h>
#include "lcd.h"

//pinii pentru adaugare pudra
#define IN1 PB3
#define IN2 PB4
#define IN3 PB5
#define IN4 PB6

//pini pentru mixer
#define MIX1 PD1
#define MIX2 PD4
#define MIX3 PD5
#define MIX4 PD6

//pompele de apa : in - introducere | out - scoatere
#define IN PD0
#define OUT PB0

//butonul de start
#define BTN PB1

int cur_step = 0;
// primesc ca input pinii si portul pentru motor
void step(int in1,int in2,int in3, int in4,volatile uint8_t *var)
{
	switch(cur_step){
		case 0:
			*var &= ~(1<<in1); //low
			*var &= ~(1<<in2); //low
			*var &= ~(1<<in3); //low
			*var|= (1<<in4); //high
			
			
			break;
		case 1:
			*var &= ~(1<<in1); //low
			*var &= ~(1<<in2); //low
			*var |= (1<<in3); //high
			*var |= (1<<in4); //high
			
			break;
		case 2:
			*var &= ~(1<<in1); //low
			*var &= ~(1<<in2); //low
			*var |= (1<<in3); //high
			*var &= ~(1<<in4); //low
			break;
		case 3:
			*var &= ~(1<<in1); //low
			*var |= (1<<in2); //high
			*var |= (1<<in3); //high
			*var &= ~(1<<in4); //low
			break;
		case 4:
			*var &= ~(1<<in1); //low
			*var |= (1<<in2); //high
			*var &= ~(1<<in3); //low
			*var &= ~(1<<in4); //low
			break;
		case 5:
			*var |= (1<<in1); //high
			*var |= (1<<in2); //high
			*var &= ~(1<<in3); //low
			*var &= ~(1<<in4); //low
			break;
		case 6:
			*var |= (1<<in1); //high
			*var &= ~(1<<in2); //low
			*var &= ~(1<<in3); //low
			*var &= ~(1<<in4); //low
			break;
		case 7:
			*var |= (1<<in1); //high
			*var &= ~(1<<in2); //low
			*var &= ~(1<<in3); //low
			*var |= (1<<in4); //high
			
			break;
		 default:
			*var &= ~(1<<in1); //low
			*var &= ~(1<<in2); //low
			*var &= ~(1<<in3); //low
			*var &= ~(1<<in4); //low
			break; 
	}
	
	cur_step = (cur_step + 1) % 8;
}

//decide in functie de comanda ce motor actionez
void stepper(int motor)
{
	int i = 0;
	int max_steps = 0;
	int i1 = 0,i2 = 0,i3 = 0,i4 = 0;
	volatile uint8_t *var = 0;
	
	//daca e 0 atunci actionez motorul pentru pudra
	if (motor == 0) {
		i1 = IN1;
		i2 = IN2;
		i3 = IN3;
		i4 = IN4;
		DDRB |= (1 << i1) | (1 << i2) | (1 << i3) | (1 << i4);
		var = &PORTB;
		max_steps = 4096;
	} else if(motor == 1){
		i1 = MIX1;
		i2 = MIX2;
		i3 = MIX3;
		i4 = MIX4;
		DDRD |= (1 << i1) | (1 << i2) | (1 << i3) | (1 << i4);
		var = &PORTD;
		max_steps = 30000;
	}
	
	//invart motor un anumit numar de pasi
	for (i = 0; i < max_steps; i++) {
		
		step(i1,i2,i3,i4,var);
		_delay_us(1000);
	}
	
	// la final pentru a nu mai consuma dezactivezi pinii
	if(motor == 0) {
		
		PORTB &= ~(1<<i1);
		PORTB &= ~(1<<i2);
		PORTB &= ~(1<<i3);
		PORTB &= ~(1<<i4);
	} else {
		PORTD &= ~(1<<i1);
		PORTD &= ~(1<<i2);
		PORTD &= ~(1<<i3);
		PORTD &= ~(1<<i4);
	}
}

// 0 - introdu apa | 1 - scoate apa
void pompa(int io)
{
	int i;
	int bit;
	volatile uint8_t *var = 0;
	
	if (io == 0) {
		DDRD |= (1 << IN);
		var = &PORTD;
		bit = IN;
	} else {
		DDRB |= (1 << OUT);
		var = &PORTB;
		bit = OUT;
	}
	
	
	*var &= ~(1 << bit);
	for ( i = 0; i < 20; i++ )
		_delay_ms(3000);
	
	
	
	*var ^= (1 << bit);
	
	
}

//in functie de etapa am 3 mesaje posibile
void lcd(int var)
{

	if (var == 0) {
		
		LCD_writeInstr(LCD_INSTR_clearDisplay);
		LCD_print("Press start");
	
	} else if(var == 1){
		LCD_writeInstr(LCD_INSTR_clearDisplay);
		//LCD_writeInstr(LCD_INSTR_nextLine);
		LCD_print("Wait for shake");
		LCD_writeInstr(LCD_INSTR_nextLine);
		LCD_print("to be prepared");
	} else if(var == 2){
		LCD_writeInstr(LCD_INSTR_clearDisplay);
		LCD_print("Finished");
		LCD_writeInstr(LCD_INSTR_nextLine);
		LCD_print("Make another one");
	}
	
	
}

// functia principala ce apeleaza toate celelalte functii
void start()
{
	
	//declar portul ca intrare
	DDRB &= ~(1 << BTN);
	// activez rezistenta de pull-up
	PORTB |= (1 << BTN);
	
	//declar ledul drept iesire
	DDRD |= (1 << PD7);
	lcd(0);
	while(1){
		
		// verific daca am apasat butonul de start
		if ((PINB & (1 << BTN)) == 0) {

				lcd(1);
				stepper(0);
				_delay_ms(1000);
				pompa(0);
				LCD_init();
				lcd(1);
				_delay_ms(2500);
				stepper(1);
				_delay_ms(1000);
				pompa(1);
				LCD_init();
				lcd(2);
				_delay_ms(1000);
				
		}
		_delay_ms(200);
	}
	
}

int main()
{
	
	LCD_init();
	LCD_writeInstr(LCD_INSTR_clearDisplay);
	
	
	start();
	
	
	
	return 0;
	
}